#pragma rtGlobals=1		// Use modern global access method.
#pragma version=1.4		// removed all of the old stuff
//
//
//	Strconstant micron ="m"			// use for Mac, I think that this works for Windows too!
//	Strconstant micron ="micron"		// use for Win
//	mu is hex B5 = 181
//	deg is hex A1 = 161
//	Windows deg is hex C1 = 193


Menu "Rotations", dynamic
	"Plot slice of 3d array",MakeGraphOfSlice()
	"put up PanelSlicer", MakePanelSlicer(sliceWave)
	Submenu "3d surface"
		"Show 3d surface",MakeGraph3dSurface()
		"autoscale z",ModifySurfer autoscale=1
		"fix z scale",Set3dSurfaceZscale()
		BeamDirectionMenuItem(0) ,/Q, SetBeamDirection(0)
		BeamDirectionMenuItem(1) ,/Q, SetBeamDirection(1)
	End
	Submenu "   Read & Interpolate"
		"2nd  Interpolate 3d grid",Interpolate3dRodriquesFromXHF(NaN)
		"1st  Read In Raw Data",readInXYZorientationsFromFile()
	End
	Submenu "   testing"
		"print a Rodriques vector",RodriquesVector2str(NaN,NaN,NaN)
		"Table of Raw Data",TableOfRawData()
		"Make test data",MakeTestData("")
	End
End
Function/S BeamDirectionMenuItem(m)			// note, "!\022(" causes a menu item to be checked and disabled
	Variable m
	Variable/G :reverse3dSurfacePlot
	NVAR flag = :reverse3dSurfacePlot
	if (m==0)
		return SelectString(flag,"!\022(","")+"beam from Right --"
	else
		return SelectString(flag,"","!\022(")+"beam from Left   --"
	endif
End
Function SetBeamDirection(flag)
	Variable flag
	NVAR rev = :reverse3dSurfacePlot
	rev = flag
	if (strlen(WinList("PanelSlicer","","WIN:64")))					// update PanleSlicer if showing
		String str = SelectString(flag,"-- beam from Left","-- beam from Right")
		Execute "PopupMenu popupBeamDirection win=PanelSlicer, value= "+"\""+str+"\""
		str = SelectString(flag,"-- beam from Right","-- beam from Left")
		PopupMenu popupBeamDirection,title=str
	endif
	SetSliceProc("", NumVarOrDefault("SliceValue",0),"","SliceValue")
End


Function/S RodriquesVector2str(x,h,f)	// print out one Rodriques vector at (X,H,F)
	Variable x,h,f
	if (numtype(x+h+f))
		x = numtype(x) ? 0 : x
		h = numtype(h) ? 0 : h
		f = numtype(f) ? 0 : f
		Prompt x, "X position in sample (m)"
		Prompt h, "H position in sample (m)"
		Prompt f, "F position in sample (m)"
		DoPrompt "position in sample", x,h,f
		if (V_flag)
			return ""
		endif
	endif
	if (numtype(x+h+f))
		if (ItemsInList(GetRTStackInfo(0))<2)
			printf "Invalid inputs XHF={%.3g, %.3g, %.3g} m\r",x,h,f
		endif
		return ""
	endif

	String str
	Wave rhat=$MakeUnique3Vector($"")		// Rodriques vector
	Wave R3dX=R3dX, R3dH=R3dH, R3dF=R3dF
	rhat[0] = Interp3d(R3dX, x,h,f)		// Rodriques vector at XHF
	rhat[1] = Interp3d(R3dH, x,h,f)
	rhat[2] = Interp3d(R3dF, x,h,f)
	rhat = (rhat[p]==0) ? 0 : rhat[p]
	Variable angle = 2*atan(normalize(rhat))*180/PI
	sprintf str, "at {%.3g, %.3g, %.3g}m,   Rodriques^={%.3g, %.3g, %.3g},  rotation angle=%.3g",x,h,f,rhat[0],rhat[1],rhat[2],angle
	if (ItemsInList(GetRTStackInfo(0))<2)
		print str
	endif
	KillWaves/Z rhat
	return str
End


Function TableOfRawData()		// put up a table of the raw data
	if (strlen(WinList("RawDataTable", "", "WIN:2")))
		DoWindow /F RawDataTable
		return 0
	endif
	Edit/W=(5,44,547,708)/K=1 XX,HH,FF,RX,RH,RF
	DoWindow /C RawDataTable
	ModifyTable width(Point)=32,format(XX)=3,sigDigits(XX)=4,format(HH)=3,sigDigits(HH)=4
	ModifyTable format(FF)=3,sigDigits(FF)=4,format(RX)=3,digits(RX)=7,sigDigits(RX)=3
	ModifyTable format(RH)=3,digits(RH)=7,sigDigits(RH)=3,format(RF)=3,digits(RF)=7
	ModifyTable sigDigits(RF)=3
	ModifyTable width(XX)=60, width(HH)=60, width(FF)=60
End



Function MakeGraphOfSlice()				// create the standard surface cut through the volume
	Wave R3dX=R3dX, R3dH=R3dH, R3dF=R3dF
	if (!WaveExists(R3dX) || !WaveExists(R3dH) || !WaveExists(R3dF))
		DoAlert 0, "MakeGraphOfSlice(), cannot find R3dX, R3dH, & R3dF"
		return 1
	endif
	if (strlen(WinList("GraphSlice", "", "WIN:1")))
		DoWindow /F GraphSlice
		return 1
	endif
	String fldr= GetDataFolder(1)			// this is the folder where everything is created and stored
	if (exists("sliceWave")!=1)			// sliceWave does not exist, yet
		Make/N=(10,10)/O sliceWave
	endif
	String noteStr = note(sliceWave)
	Display /W=(466,47,1016,660)/K=1 
	DoWindow /C GraphSlice
	AppendMatrixContour sliceWave

	String str = fldr+"SliceValue"
	Variable/G $str							// ensure existance of SliceValue
	NVAR SliceValue=$str

	String/G $(fldr+"SliceAngleAxis")		// ensure existance of SliceAngleAxis, the name of the rotation axis
	SVAR SliceAngleAxis=$(fldr+"SliceAngleAxis")
	SliceAngleAxis = SelectString(strlen(SliceAngleAxis),"X",SliceAngleAxis)

	String/G $( fldr+"SliceNormal")		// ensure existance of SliceNormal
	SVAR SliceNormal=$( fldr+"SliceNormal")
	SliceNormal = SelectString(strlen(SliceNormal),"H",SliceNormal)

	Variable/G $(fldr+"phiSlice")			// ensure existance of phiSlice
	NVAR phi=$(fldr+"phiSlice")

	Variable/G $(fldr+"CutResolution")		// ensure existance of resolution
	NVAR CutResolution=$(fldr+"CutResolution")
	CutResolution = (CutResolution>0.0) ? CutResolution : 1.0

	ModifyContour sliceWave autoLevels={*,*,5}, rgbLines=(0,0,0)		// ModifyContour sliceWave autoLevels={0,0.001,11}, rgbLines=(0,0,0)
	ModifyContour sliceWave labelBkg=1
	AppendImage sliceWave
	ModifyImage sliceWave ctab= {*,*,RedWhiteBlue,0}					// red is negative, white=0, blue is postive
	ModifyGraph gfMult=130
	ModifyGraph mirror=1,minor=1
	ModifyGraph/Z lSize('sliceWave=0')=3
	ModifyGraph zero=2
	Label left StringByKey("YaxisName",noteStr,"=")+" (\\U)"
	Label bottom StringByKey("XaxisName",noteStr,"=")+" (\\U)"
//	SetAxis/A/R left
	SetAxis/A left
	SetAxis/A bottom
	Cursor/P/I A sliceWave 1,1
	ShowInfo
	TextBox/N=titleText/F=0/S=3/A=LT/X=3.06/Y=2.06 "lattice tilt (radians)\r"+GetWavesDataFolder(sliceWave,2)

	ColorScale/N=colorScale1/A=RC/X=-4.75/Y=0 image=sliceWave, heightPct=30
	ColorScale/C/N=colorScale1 lowTrip=0.001,lblMargin=0
	ColorScale/C/N=colorScale1 "radian"

	CutSlicePopUpProc("",0,"")					// force an update using current values
	DoUpdate
	MakePanelSlicer(sliceWave)					// put up the panel with controls
End


Function MakeGraph3dSurface()					// make a 3-d surface to go with cut
	// Mac V. 3.0.0
	if (exists("CreateSurfer") !=4)		// Do nothing if the Surface Plotter XOP is not available.
		DoAlert 0, "Surface Plotter XOP must be installed"
		return 1
	endif
	if (exists("sliceWaveAbs")!=1 || exists("sliceWaveLog")!=1)
		DoAlert 0, "sliceWaveAbs or  sliceWaveLog does not exists, try MakeGraphOfSlice() first"
		return 1
	endif
	Execute "CreateSurfer/K=1"
	MoveWindow 7,53,675,571 		//	left, top, right, bottom
	Execute "ModifySurfer  FactoryDefaults, Update=0"
	Execute "ModifySurfer expandWindow=0"
	Execute "ModifySurfer srcWave=sliceWaveAbs"
	Execute "ModifySurfer secondWave=sliceWaveLog"
	Execute "ModifySurfer surfaceColorWave=sliceWaveLog"
	Execute "ModifySurfer  srcType=1,plotType=3,  setControlView=3"
	Execute "ModifySurfer  theta=28.9,  phi=311.8,  zScale=1,  xStep=1,  yStep=1"
	Execute "ModifySurfer  frame=895,  drawFrame=0, drawBox=1"
	Execute "ModifySurfer  numXTicks=2,  numYTicks=2,  numZTicks=2, drawTicks=5"
	Execute "ModifySurfer  noTicksX=0,  noTicksY=0,  noTicksZ=0"
	Execute "ModifySurfer palette=RedWhiteBlue, reverseColorMap=1"
	Execute "ModifySurfer  xmin=NaN,  xmax=NaN,  ymin=NaN,  ymax=NaN"
	Execute "ModifySurfer  surfaceColorFill=1,  grids=1"
	Execute "ModifySurfer  numContourLevels=12,  contour=1027"
	Execute "ModifySurfer  marker=16,  markerSize=1,  markerColorType=4"
	Execute "ModifySurfer  scatterDepthCue=1, imageType=1, BPP=16"
	Execute "ModifySurfer  Update=1"
	Execute "ModifySurfer fullUpdate"
End
Function Set3dSurfaceZscale()
	if (exists("GetSurfer") !=4)		// Do nothing if the Surface Plotter XOP is not available.
		DoAlert 0, "Surface Plotter XOP must be installed"
		return 1
	endif
	Execute "GetSurfer surferName"			// store name in S_SurferName
	SVAR S_SurferName=S_SurferName
	if (strlen(S_SurferName)<=1)			// no surface plot is up
		return 1
	endif

	WaveStats/Q/M=1 sliceWaveAbs
	Execute "ModifySurfer zmin=0,zmax="+num2str(V_max)
	Execute "ModifySurfer autoscale=0"
End



Function MakePanelSlicer(sliceW) : Panel	// panel to go with GraphSlice
	Wave sliceW
	if (!WaveExists(sliceW))
		DoAlert 0, "MakePanelSlicer(), cannot put up panel by hand, it goes up automatically with MakeGraphOfSlice()"
		return 1
	endif
	if (strlen(WinList("GraphSlice",";","WIN:1"))<=0)
		DoAlert 0, "MakePanelSlicer(), needs to have GraphSlice already up, it goes up automatically with MakeGraphOfSlice()"
		return 1
	endif

	String fldr= GetWavesDataFolder(sliceW,1)		// this is the folder where everything is created and stored
	String noteStr = note(sliceWave)

	String str = fldr+"SliceValue"
	Variable/G $str							// ensure existance of SliceValue
	NVAR SliceValue=$str
	String/G $( fldr+"SliceNormal")		// ensure existance of SliceNormal
	SVAR SliceNormal=$( fldr+"SliceNormal")
	SliceNormal = SelectString(strlen(SliceNormal),"H",SliceNormal)
	String/G $(fldr+"SliceAngleAxis")		// ensure existance of SliceAngleAxis, the name of the rotation axis
	SVAR SliceAngleAxis=$(fldr+"SliceAngleAxis")
	SliceAngleAxis = SelectString(strlen(SliceAngleAxis),"X",SliceAngleAxis)
	Variable/G $(fldr+"phiSlice")			// ensure existance of phiSlice
	NVAR phi=$(fldr+"phiSlice")
	Variable/G $(fldr+"CutResolution")		// ensure existance of resolution
	NVAR CutResolution=$(fldr+"CutResolution")
	CutResolution = (CutResolution>0.0) ? CutResolution : 1.0

	if (strlen(WinList("PanelSlicer","","WIN:64")))
		DoWindow/K PanelSlicer							// kill panel if it exists
	endif
	NewPanel /W=(229,44,449,182)/K=1				// W=(left, top, right, bottom )
	DoWindow /C PanelSlicer
	AutoPositionWindow /M=0/R=GraphSlice PanelSlicer	// position panel by top most gizmo
	Variable mode, lo=ceil(NumberByKey("Hlo",noteStr,"=")), hi=floor(NumberByKey("Hhi",noteStr,"="))
	SetVariable setvarSlice,pos={1,2},size={75,18},proc=SetSliceProc,title=SliceNormal
	SetVariable setvarSlice,fSize=12
	SetVariable setvarSlice,limits={lo,hi,1},value= SliceValue
	SetVariable setvarPhi,pos={141,2},size={71,18},disable=1,proc=SetPhiCutProc,title="phi"
	SetVariable setvarPhi,fSize=12,limits={0,180,0},value=phiSlice
	SetVariable setvarResolution,pos={9,25},size={145,18},proc=SetResolutionProc,title="resolution (m)"
	SetVariable setvarResolution,fSize=12,limits={0,inf,0},value=CutResolution
	mode = WhichListItem(SliceNormal,"X;H;F;Y;Z;phiZ")+1
	PopupMenu CutNormalPopUp,pos={81,1},size={35,20},proc=CutSlicePopUpProc
	PopupMenu CutNormalPopUp,mode=mode,popvalue=SliceNormal,value= #"\"X;H;F;Y;Z;phiZ\""
	SetVariable setvarPhi disable=!stringmatch(SliceNormal,"phiZ")
	mode = WhichListItem(SliceAngleAxis,"X;H;F;Y;Z;R_Cyl;Phi_Cyl;Axis_Cyl;Total")+1
	PopupMenu CutAnglePopUp,pos={10,48},size={135,20},proc=CutSlicePopUpProc,title="rotation axis"
	PopupMenu CutAnglePopUp,mode=mode,popvalue=SliceAngleAxis,value= #"\"X;H;F;Y;Z;R_Cyl;Phi_Cyl;Axis_Cyl;Total;\""
	PopupMenu popup3dSurface,pos={14,84},size={89,20},proc=PopMenu3dSurfaceProc,title="3d surface"
	PopupMenu popup3dSurface,fSize=12,mode=0,value= #"\"Show 3d surface;autoscale z;fix z scale\""
	PopupMenu popupBeamDirection,pos={14,110},size={89,20},proc=BeamDirectionProc,fSize=12,mode=0
	NVAR flag = :reverse3dSurfacePlot
	BeamDirectionProc("",flag+1,"")
End
Function PopMenu3dSurfaceProc(ctrlName,popNum,popStr) : PopupMenuControl	//control for 3d-surface view
	String ctrlName
	Variable popNum
	String popStr
	strswitch(popStr)
		case "Show 3d surface":
			MakeGraph3dSurface()
			break
		case "autoscale z":
			Execute "ModifySurfer autoscale=1"
			break
		case "fix z scale":
			Set3dSurfaceZscale()
			break
	endswitch
	return 0
End
Function BeamDirectionProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	SetBeamDirection(strsearch(popStr,"left",0,2)>=0)
	return 0
End



// this updates the slice surface.  It is called each time you want a new level.  To change the kind of slice (x, y, or z) call CutSlicePopUpProc
Function SetSliceProc(ctrlName,value,varStr,varName) : SetVariableControl
	String ctrlName
	Variable value
	String varStr
	String varName
	if (cmpstr("SliceValue",varName))
		Abort "SetSliceProc only callable from the button"
	endif

	Wave sliceW = ImageNameToWaveRef("", "sliceWave" )
	String fldr = GetWavesDataFolder(sliceW,1)
	NVAR CutResolution=$(fldr+"CutResolution")
	NVAR SliceValue=$(fldr+"SliceValue")
	SVAR SliceNormal=$(fldr+"SliceNormal")						// name of normal to the slice
	SVAR SliceAngleAxis=$(fldr+"SliceAngleAxis")					// name of desired rotation axis

	Wave surfNormal = $MakeDirectionVector(SliceNormal)
	SlicePlaneIn3d(CutResolution,surfNormal,SliceValue,SliceAngleAxis)	// creates or recalcs sliceWave

	String noteStr = note(sliceW)
	noteStr = ReplaceNumberByKey("SliceValue",noteStr,SliceValue,"=")
	Note/K sliceW
	Note sliceW, noteStr
	KillWaves/Z surfNormal

	String wName = SquaredVersionOfSurf(sliceW)					// squared version of sliceWave
	Wave sliceSquare=$wName
	if (NumVarOrDefault(":reverse3dSurfacePlot",0))
		Variable xlo=DimOffset(sliceSquare,0), N=DimSize(sliceSquare,0), dx=DimDelta(sliceSquare,0)
		String xstr = GetDimLabel(sliceSquare,0,-1)
		ImageTransform flipRows sliceSquare
		SetScale/I x ((N-1)*dx+xlo),xlo,xstr, sliceSquare
	endif
	Duplicate/O sliceSquare, sliceWaveAbs, sliceWaveLog			// these waves needed to make the 3d surface plot
	sliceWaveAbs = abs(sliceSquare)
	sliceWaveLog = log(abs(sliceSquare)) * sign(sliceSquare)
	sliceWaveLog = numtype(sliceWaveLog) ? 0 : sliceWaveLog
	KillWaves/Z sliceSquare

	// set range of color to be symmetric if it was not set by the user (this is a little tricky)
	String str = StringByKey("RECREATION",ImageInfo("","sliceWave",0))	//ctab= {*,*,RedWhiteBlue,0}
	str = StringByKey("ctab",str,"=")
	str = str[2,strlen(str)-2]
	if (strlen(StringFromList(0,str,",")+StringFromList(1,str,","))>27 || strsearch(str,"*,",0)>=0)	// probably auto-scaled
		WaveStats/Q/M=1 sliceW				// these 3 lines make red negative, blue positive
		Variable colRange = max(abs(V_min),abs(V_max))
		colRange = (colRange==0||numtype(colRange)) ? sqrt(2)*1e-4 : colRange
		ModifyImage sliceWave ctab= {-colRange,colRange,RedWhiteBlue,0}	// red is negative, white=0, blue is postive
	endif

	// update the title for the plot
	NVAR phi=$(fldr+"phiSlice")
	String listAxis = "X:X;H:H;F:F;Y:Y;Z:Z;R_Cyl:r\\X0\\S^\\M;Phi_Cyl:\\F'Symbol'f\\F]0\\X0\\S^\\M;Axis_Cyl:axis;Total:total angle"
	TextBox/C/N=titleText "lattice tilt (radians)"
	AppendText/N=titleText fldr
	if (stringmatch(SliceNormal,"phiZ"))
		sprintf str "Surface normal toward phi=%g",phi
	else
		sprintf str "Surface normal toward %s",SliceNormal
	endif
	AppendText/N=titleText str
	sprintf str "cut perp to %s at %g m from origin",SliceNormal,SliceValue
	AppendText/N=titleText str
	if (stringmatch(SliceAngleAxis,"Total"))
		sprintf str "total rotaion angle\r"
	else
		sprintf str "rotation axis is along \\[0%s direction",StringByKey(SliceAngleAxis,listAxis)
	endif
	AppendText/N=titleText str

	KillWaves/Z surfNormal_
	return 0
End
Function/S SquaredVersionOfSurf(wav)		// return the name of a new wave that is the squared version of the input
	Wave wav								// a 2d wave that needs to be squared
	Variable iN=DimSize(wav,0), jN=DimSize(wav,1)	// dimenstions of the input wave
	Variable i,j
	String wName, noteStr

	WaveStats/Q/M=1 wav
	if ((V_npnts+V_numINFs)<=0)						// no valid points, just return one the same size
		wName = UniqueName("surface_",1,0)
		i = max(iN,jN)
		Make/N=(i,i) $wName
		Wave surf = $wName
		noteStr = note(wav)
		Note surf, noteStr
		CopyScales/P wav, surf
		return GetWavesDataFolder(surf,2)
	endif

	wName = UniqueName("vec_",1,0)
	Make/N=(iN)/D $wName
	Wave veci = $wName
	wName = UniqueName("vec_",1,0)
	Make/N=(jN)/D $wName
	Wave vecj = $wName

	// find the first i, that has a valid number (is not NaN)
	Variable dx=DimDelta(wav,0), dy=DimDelta(wav,1)
	Variable ilo, jlo, ihi, jhi							// new size of surace
	for (jlo=0;jlo<jN;jlo+=1)
		veci = wav[p][jlo]
		WaveStats/Q/M=1 veci
		if (V_numNans<iN)
			break
		endif
	endfor
	for (ilo=0;ilo<iN;ilo+=1)
		vecj = wav[ilo][p]
		WaveStats/Q/M=1 vecj
		if (V_numNans<jN)
			break
		endif
	endfor
	for (jhi=jN-1;jhi>=0;jhi-=1)
		veci = wav[p][jhi]
		WaveStats/Q/M=1 veci
		if (V_numNans<iN)
			break
		endif
	endfor
	for (ihi=iN-1;ihi>=0;ihi-=1)
		vecj = wav[ihi][p]
		WaveStats/Q/M=1 vecj
		if (V_numNans<jN)
			break
		endif
	endfor

	Variable iNew, jNew, x0,y0
	Variable size = max(abs((jhi-jlo)*dy), abs((ihi-ilo)*dx))
	iNew = ceil(size/dx)+1				// new dimension in i
	jNew = ceil(size/dy)+1				// new dimension in j
	x0 = ilo*dx+DimOffset(wav,0)
	y0 = jlo*dy+DimOffset(wav,1)
	//	printf "i=[%d,%d],   j=[%d,%d]\r",ilo,ihi,jlo,jhi
	//	printf "size=%g,   x0=%g,  y0=%g,  iNew=%d,  jNew=%d\r",size,x0,y0,iNew,jNew
	wName = UniqueName("surface_",1,0)
	Make/N=(iNew,jNew) $wName
	Wave surf = $wName
	surf = NaN
	CopyScales/P wav, surf
	SetScale/P x, x0, dx, surf
	SetScale/P y, y0, dy, surf
	surf[0,ihi-ilo][0,jhi-jlo] = wav[ilo+p][jlo+q]	// set the values
	noteStr = note(wav)
	Note surf, noteStr
	KillWaves/Z veci, vecj
	return GetWavesDataFolder(surf,2)		// return full path for assingment to a wave
End



// this is called by the popup that selects the direction of the cut and the direction of the angle for sliceWave
Function CutSlicePopUpProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr

	Wave sliceW = ImageNameToWaveRef("", "sliceWave" )		// the 2-d wave displayed as an image and contours
	String fldr = GetWavesDataFolder(sliceW,1)
	NVAR SliceValue=$(fldr+"SliceValue")
	SVAR SliceNormal=$(fldr+"SliceNormal")					// name of normal to sliceWave
	SVAR SliceAngleAxis=$(fldr+"SliceAngleAxis")				// name of desired rotation axis

	if (stringmatch(ctrlName,"CutNormalPopUp"))				// called by a change to the normal to sliceWave
		SliceNormal = popStr									// set the name of the normal to sliceWave
		SetVariable setvarPhi disable=!stringmatch(SliceNormal,"phiZ")
	elseif (stringmatch(ctrlName,"CutAnglePopUp"))			// called by a change to the requested rotaion axis {X,H,F,Y,Z,phi,axis,total}
		SliceAngleAxis = popStr									// name of desired rotation axis
	endif

	SetSliceProc("",SliceValue,"","SliceValue")					// force recalculation
	String noteStr = note(sliceW)
	Label left StringByKey("YaxisName",noteStr,"=")+" (\\U)"
	Label bottom StringByKey("XaxisName",noteStr,"=")+" (\\U)"
	DoUpdate					// this is needed to refresh values for call to FixAspectRatio()
	FixAspectRatio("GraphSlice")

	Variable lo=NumberByKey("nlo",noteStr,"="), hi=NumberByKey("nhi",noteStr,"=")
	if (strlen(WinList("PanelSlicer","","WIN:64"))>1)
		SetVariable setvarSlice,win=PanelSlicer,limits={lo,hi,1}
		SetVariable setvarSlice,win=PanelSlicer,title=SelectString(stringmatch(SliceNormal,"phiZ"),SliceNormal,"r")
	endif
End
Function FixAspectRatio(graphName)		// set aspect ratio of graph to match the axes
	String graphName
	if (strlen(graphName)<1)
		graphName = StringFromList(0,WinList("*",";","WIN:1"))
	endif
	if (WinType(graphName)!=1)
		DoAlert 0, "FixAspectRatio(), unable to get size of bottom axis"
		return 1
	endif
	GetAxis /W=$graphName/Q bottom
	if (V_flag)
		DoAlert 0, "FixAspectRatio(), unable to get size of bottom axis"
		return 1
	endif
	Variable xRange=abs(V_max-V_min)
	GetAxis /W=$graphName/Q left
	if (V_flag)
		DoAlert 0, "FixAspectRatio(), unable to get size of left axis"
		return 1
	endif
	Variable yRange=abs(V_max-V_min)
	Variable aspect = yRange/xRange
	//	printf "xRange=%g, yRange=%g,  aspect=%g\r",xRange,yRange,aspect
	if (aspect>=1)
		ModifyGraph width={Aspect,1/aspect},height=0		// tall & thin
	else
		ModifyGraph height={Aspect,aspect},width=0		// short & fat 		
	endif
End
Function SetPhiCutProc(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	CutSlicePopUpProc("",0,"")
//	XHFPopMenuProc("",-1,"phi")
End
Function SetResolutionProc(ctrlName,varNum,varStr,varName) : SetVariableControl // proc for CutResolution
	String ctrlName
	Variable varNum
	String varStr
	String varName
	if (varNum<=0)
		DoAlert 0, "you set the resolution to  0, what were you thinking?"
	elseif (abs(varNum)<0.1)
		DoAlert 0, "Caution, this fine a resolution will take a long (really long) time to interpolate!"
	endif
End



Function/S SlicePlaneIn3d(resolution,normalW,SliceValue,rotation)	// calculate a cut, result goes into SliceWave
	Variable resolution			// resolution, grid spacing of output wave (m)
	Wave normalW				// surface normal of cut plane, (need not be of length 1)
	Variable SliceValue			// intercept displaced along normal
	String rotation				// specifys the desired rotation axis, can be one of "X;H;F;Y;Z;R_Cyl;Phi_Cyl;Axis_Cyl;Total"

	Wave R3dX=R3dX, R3dH=R3dH, R3dF=R3dF
	String rotations="X;H;F;Y;Z;R_Cyl;Phi_Cyl;Axis_Cyl;Total", errstr=""
	if (!WaveExists(R3dX) || !WaveExists(R3dH) || !WaveExists(R3dF))
		errstr = "Error in SlicePlaneIn3d(), R3dX or R3dH or R3dF absent"
	elseif (!WaveExists(normalW))
		errstr = "SlicePlaneIn3d(), normal does not exists"
	elseif (numtype(norm(normalW)))
		errstr = "SlicePlaneIn3d(), normal is invalid"
	elseif (norm(normalW)<1e-10)
		errstr = "SlicePlaneIn3d(), normal is too short"
	elseif (!(resolution>0))
		errstr = "SlicePlaneIn3d(), resolution = "+num2str(resolution)
	elseif (numtype(SliceValue))
		errstr = "SlicePlaneIn3d(), SliceValue = "+num2str(SliceValue)
	elseif (WhichListItem(rotation,rotations)<0)
		errstr = "SlicePlaneIn3d(), rotation = '"+rotation+"'"
	endif
	if (strlen(errstr))
		DoAlert 0, errstr
		return ""
	endif

	Wave normal=$MakeUnique3Vector(normalW)		// make a new 3-vector, and optionally set it to preset[]
	normalize(normal)

	Variable calcAxis=0	
	if (!stringmatch(rotation,"Total"))					// need to compute angle component
		Wave rotationAxis = $MakeDirectionVector(rotation)// want rotation component about rotationAxis[]
		calcAxis = numtype(rotationAxis[0])!=0		// axis of rotation is not constant, needs to be re-calculated for each point
	endif
	String noteStr = note(R3dX)
	Variable x0,h0,f0, dXaxis, dHaxis, dFaxis
	x0 = NumVarOrDefault(":X0",NumberByKey("X0",noteStr,"=")) //origin to work from (especially important for cylindrical coordinates)
	h0 = NumVarOrDefault(":H0",NumberByKey("H0",noteStr,"="))
	f0 = NumVarOrDefault(":F0",NumberByKey("F0",noteStr,"="))
	dXaxis = NumVarOrDefault(":dXaxis",NumberByKey("dX",noteStr,"=")) // direction of the axis for cylindrical coordinates
	dHaxis = NumVarOrDefault(":dHaxis",NumberByKey("dH",noteStr,"="))
	dFaxis = NumVarOrDefault(":dFaxis",NumberByKey("dF",noteStr,"="))
	Wave axis=$MakeUnique3Vector($"")				// axis of cylindrical coords, needed to calculate rotationAxis[]
	axis = {dXaxis,dHaxis,dFaxis}						// axis is assumed to go through xhf0[]
	normalize(axis)

	Variable xlo,xhi, hlo,hhi, flo, fhi					// extent of box in x,h,f
	xlo = NumberByKey("Xlo",noteStr,"=")
	xhi = NumberByKey("Xhi",noteStr,"=")
	hlo = NumberByKey("Hlo",noteStr,"=")
	hhi = NumberByKey("Hhi",noteStr,"=")
	flo = NumberByKey("Flo",noteStr,"=")
	fhi = NumberByKey("Fhi",noteStr,"=")

	//	Need point representing center of cut surface.  It is:     xhf0[] + normal[]*SliceValue
	//	We are assuming that xhf0[] in the 3d volume is the origin that we want.
	// 	Then need x and y directions in the slice, this depends upon the surfNormal[]
	//	call these directions sx and sy, along the x and y in the slice
	//
	// get the sx and sy axes, the 3vectors giving x, y directions for the slice, and the center of the slice
	Wave xhf0=$MakeUnique3Vector($"")		// origin
	Wave hat=$MakeUnique3Vector($"")		// approximate direction of x-axis
	Wave sx=$MakeUnique3Vector($"")			// direction of x-axis in surface
	Wave sy=$MakeUnique3Vector($"")			// direction of y-axis in surface
	Wave center=$MakeUnique3Vector($"")		// center of the surface
	Wave xhf=$MakeUnique3Vector($"")		// a point on surface in XHF space
	Wave Rod=$MakeUnique3Vector($"")		// direction of Rodriques vector
	Wave rvec=$MakeUnique3Vector($"")		// vector from xhf0[] to point on slice
	Wave rhat=$MakeUnique3Vector($"")		// like xhf, but axial part removed

	xhf0 =  {x0,h0,f0}
	center =xhf0[p] + SliceValue*normal[p]// center[] = {X0,H0,F0} + SliceValue*normal[], center of slice
	if (abs(normal[0])<=abs(normal[1]))
		hat = {1,0,0}						// use xhat to define sx
	else	
		hat = {0,1,0}						// use yhat to define sx
	endif
	Variable dot
	dot = MatrixDot(normal,hat)
	sx = hat[p] - dot*normal[p]			// sx is unit vector perpendicular to normal, toward hat direction
	normalize(sx)
	Cross normal, hat
	Wave W_Cross=W_Cross
	sy = W_Cross
	normalize(sy)

	// next what is the range of the plane, what are its starting and stopping values along sx and sy
	Variable sxlo, sxhi, sylo, syhi,snlo,snhi	// range of plane, used to set the scaling, use cube corners as limits (snlo,snhi refers to distance along normal)
	Variable nsx,nsy						// sliceWave[nsx][nsy]
	FindRangeForBox(xlo,xhi,hlo,hhi,flo,fhi,sx,xhf0,sxlo,sxhi)		// determine max range along sx[] (relative to xhf0[])
	FindRangeForBox(xlo,xhi,hlo,hhi,flo,fhi,sy,xhf0,sylo,syhi)		// determine max range along sx[] (relative to xhf0[])
	FindRangeForBox(xlo,xhi,hlo,hhi,flo,fhi,normal,xhf0,snlo,snhi)// determine max range along normal[] (relative to xhf0[])
	nsx = ceil((sxhi-sxlo)/resolution)+1			// number of points in sliceWave
	nsy = ceil((syhi-sylo)/resolution)+1
	if (ItemsInList(GetRTStackInfo(0))<2)
		printf "SlicePlaneIn3d(resolution=%g, surfNorm={%.3g, %.3g, %.3g}, SliceValue=%g, rotationAxis='%s')\r",resolution,normalW[0],normalW[1],normalW[2],SliceValue,rotation
		printf "the range of slice is:     sx = [%g, %g] (%d pts),  sy=[%g, %g] (%d pts),  along normal = [%.4g, %.4g]\r",sxlo,sxhi,nsx,sylo,syhi,nsy,snlo,snhi
		printf "normal={%.3g, %.3g, %.3g},     sx={%.3g, %.3g, %.3g},     sy={%.3g, %.3g, %.3g}\r",normal[0],normal[1],normal[2],sx[0],sx[1],sx[2],sy[0],sy[1],sy[2]
		printf "xhf0={%.3g, %.3g, %.3g},     axis={%.3g, %.3g, %.3g}\r",xhf0[0],xhf0[1],xhf0[2],axis[0],axis[1],axis[2]
	endif
	noteStr = ReplaceNumberByKey("nlo",noteStr,snlo,"=")
	noteStr = ReplaceNumberByKey("nhi",noteStr,snhi,"=")
	Make/N=(nsx,nsy)/O sliceWave
	SetScale/I x sxlo,sxhi,"m", sliceWave
	SetScale/I y sylo,syhi,"m", sliceWave
	noteStr = ReplaceNumberByKey("SliceValue",noteStr,SliceValue,"=")
	noteStr = ReplaceStringByKey("XaxisName",noteStr,axisName(sx),"=")
	noteStr = ReplaceStringByKey("YaxisName",noteStr,axisName(sy),"=")
	Note/K sliceWave
	Note sliceWave, noteStr

	// next fill the sliceWave
	Variable i,j, xx,yy, angle
	for (j=0;j<nsy;j+=1)
		yy = DimOffset(sliceWave,1) + j*DimDelta(sliceWave,1)
		for (i=0;i<nsx;i+=1)
			xx = DimOffset(sliceWave,0) + i*DimDelta(sliceWave,0)
			xhf = center[p] + xx*sx[p] + yy*sy[p]					// point in XHF space corresponding to point (i,j) in slice
			// compute the angle for this point and save it in sliceWave[i][j]
			Rod[0] = Interp3d(R3dX, xhf[0],xhf[1],xhf[2])		// Rodriques vector at xhf
			Rod[1] = Interp3d(R3dH, xhf[0],xhf[1],xhf[2])
			Rod[2] = Interp3d(R3dF, xhf[0],xhf[1],xhf[2])
			angle = 2*atan(normalize(Rod))						// Rodriques angle (radians), Rod is now normalized
			if (calcAxis)											// need to recalculate rotationAxis[] each time
				rvec = xhf - xhf0									// rvec is now direction of point relative to the origin of cylinder
				dot = MatrixDot(axis,rvec)
				rhat = rvec - dot*axis								// put rhat in the plane where z along axis is 0, only r & phi
				normalize(rhat)
				if (stringmatch(rotation,"R_Cyl"))					// rotation around axis sticking out radially from axis, r^ direction
					rotationAxis = rhat
				elseif (stringmatch(rotation,"Phi_Cyl"))			// rotation around axis in phi^ direction, probably the biggest for an indent
					Cross axis, rhat									// rotationAxis = axis x rHat - phi^
					rotationAxis = W_Cross
				elseif (stringmatch(rotation,"Axis_Cyl"))			// rotation around axis of cylinder, circulating direction
					rotationAxis = axis
				else
					Abort "SlicePlaneIn3d(), invalid rotation"		// should not be possible to reach this line
				endif
			endif

			if (i==floor(nsx/2) && j==floor(nsy/2) && ItemsInList(GetRTStackInfo(0))<2)
				printf "at center of sliceWave[%d][%d],      angle=%.3g   Rod={%.3g, %.3g, %.3g},      XHF={%.3g, %.3g, %.3g}",i,j,angle*180/PI,Rod[0],Rod[1],Rod[2],xhf[0],xhf[1],xhf[2]
				if (WaveExists(rotationAxis))
					printf ",      rotAxis={%.3g, %.3g, %.3g}",rotationAxis[0],rotationAxis[1],rotationAxis[2]
				endif
				if (calcAxis)
					printf ",      rvec={%.4g, %.4g, %.4g},       rhat={%.4g, %.4g, %.4g}",rvec[0],rvec[1],rvec[2],rhat[0],rhat[1],rhat[2]
				endif
				print ""
			endif

			dot = stringmatch(rotation,"Total") ? 1 : MatrixDot(rotationAxis,Rod)
			sliceWave[i][j] = angle * dot							// amount of rotation about rotationAxis[]
		endfor
	endfor

	KillWaves/Z rotationAxis,axis,normal,Rod,xhf0,xhf,center,sx,sy,hat,rhat,rvec,W_Cross
	return GetWavesDataFolder(sliceWave,2)
End


// Find the range [lo,hi] along direction vec[] through point v0[] for each corner of a box.
// So (v0[]+lo*n[]), (v0[]+hi*n[]), just span range of box
Static Function FindRangeForBox(xlo,xhi,ylo,yhi,zlo,zhi,vec,v0,lo,hi)
	Variable xlo,xhi,ylo,yhi,zlo,zhi
	Wave vec						// direction of axis
	Wave v0						// location of origin
	Variable &lo, &hi				// the values retrned

	Variable dot
	Wave xyz=$MakeUnique3Vector($"")			// xyz holds the position of each of the box corners
	Wave n=$MakeUnique3Vector($"")				// n is normalized version of vec
	n = vec
	normalize(n)

	lo = Inf	;	hi = -Inf
	// check each of the 8 corners
	xyz = {xlo,ylo,zlo}	;	xyz -= v0	;	dot = MatrixDot(xyz,n)	;	hi = max(hi,dot)	;	lo = min(lo,dot)
	xyz = {xhi,ylo,zlo}	;	xyz -= v0	;	dot = MatrixDot(xyz,n)	;	hi = max(hi,dot)	;	lo = min(lo,dot)
	xyz = {xlo,yhi,zlo}	;	xyz -= v0	;	dot = MatrixDot(xyz,n)	;	hi = max(hi,dot)	;	lo = min(lo,dot)
	xyz = {xhi,yhi,zlo}	;	xyz -= v0	;	dot = MatrixDot(xyz,n)	;	hi = max(hi,dot)	;	lo = min(lo,dot)
	xyz = {xlo,ylo,zhi}	;	xyz -= v0	;	dot = MatrixDot(xyz,n)	;	hi = max(hi,dot)	;	lo = min(lo,dot)
	xyz = {xhi,ylo,zhi}	;	xyz -= v0	;	dot = MatrixDot(xyz,n)	;	hi = max(hi,dot)	;	lo = min(lo,dot)
	xyz = {xlo,yhi,zhi}	;	xyz -= v0	;	dot = MatrixDot(xyz,n)	;	hi = max(hi,dot)	;	lo = min(lo,dot)
	xyz = {xhi,yhi,zhi}	;	xyz -= v0	;	dot = MatrixDot(xyz,n)	;	hi = max(hi,dot)	;	lo = min(lo,dot)
	KillWaves/Z n, xyz
End


// Used to make temporary internal waves, a typical use would be:    Wave xxx=$MakeUnique3Vector($"")
// You will probably want to KillWaves/Z xxx, at the end of the routine.
Function/S MakeUnique3Vector(preset)		// make a new 3-vector, and optionally set it to preset[]
	Wave preset							// optional wave, if none call with $""
	String wName = UniqueName("vec3_",1,0)
	Make/N=3/D $wName
	Wave vec=$wName
	if (WaveExists(preset))
		vec = preset
	endif
	return GetWavesDataFolder(vec,2)		// return full path for assingment to a wave
End




// The big interpolation from random (XYZ)&Rodriques, to three grids of RX, RH, RF
Function Interpolate3dRodriquesFromXHF(resolution)
	Variable resolution	// resolution in final array (m)

	Wave XX,HH,FF		// positions (beamline coords) for the (RX,RH,RF)
	Wave RX,RH,RF		// three components of the Rodriques vector
	Wave totalAngles
	if (!WaveExists(XX) || !WaveExists(HH) || !WaveExists(FF) || !WaveExists(RX) || !WaveExists(RH) || !WaveExists(RF))
		DoAlert 0, "the waves XX,HH,FF, RX,RH,RF cannot all be found, are you in the right data folder?"
		return 1
	endif
	Variable N=numpnts(RX)

	Variable defaultResolution = NumVarOrDefault("CutResolution",1.0)
	defaultResolution = defaultResolution<=0 ? NaN : defaultResolution
	resolution = (resolution<=0) ? NaN : resolution			// force 0 or negative to be invalid
	if (numtype(resolution) || resolution<=0)
		Prompt resolution, "resolution in final array (m)"
		resolution = numtype(resolution) ? defaultResolution : max(abs(resolution),resolution)
		DoPrompt "set resolution", resolution
		if (V_flag)
			return 1
		endif
	endif
	if (numtype(resolution) || resolution<=0)
		DoAlert 0, "a resolution of "+num2str(resolution)+" is not valid"
		return 1
	endif

	Variable threshold = 0.18							// (m) a motion of the PM500 greater than this is intentional (less is jitter)
	Variable Xoff,dX, dimX=1							// for setting scaling in X-dimension
	Variable Hoff,dH, dimH=1
	Variable Foff,dF, dimF=1
	FindScalingFromVec(XX,threshold,Xoff,dX,dimX)
	FindScalingFromVec(HH,threshold,Hoff,dH,dimH)
	FindScalingFromVec(FF,threshold,Foff,dF,dimF)
	//	printf "X=[%g,%g], dimX=%d\r",Xoff,Xoff+dX*(dimX-1),dimX
	//	printf "H=[%g,%g], dimH=%d\r",Hoff,Hoff+dH*(dimH-1),dimH
	//	printf "F=[%g,%g], dimF=%d\r",Foff,Foff+dF*(dimF-1),dimF

	Variable Xlo, Hlo, Flo								// low end of 3d volume
	Variable Xhi, Hhi, Fhi								// high end of 3d volume
	Variable NX, NH, NF									// number of points (not intervals) along each axis

	Xlo = resolution*floor(Xoff/resolution)				// low end of 3d volume
	Hlo = resolution*floor(Hoff/resolution)
	Flo = resolution*floor(Foff/resolution)

	NX=ceil((dimX*dX+Xoff-Xlo)/resolution)			// number of points (not intervals) along each axis
	NH=ceil((dimH*dH+Hoff-Hlo)/resolution)
	NF=ceil((dimF*dF+Foff-Flo)/resolution)
	NX = max(NX,1)									// have to have at least one point (no intervals)
	NH = max(NH,1)
	NF = max(NF,1)

	Xhi = Xlo + (NX-1)*resolution						// high end of 3d volume
	Hhi = Hlo + (NH-1)*resolution
	Fhi = Flo + (NF-1)*resolution

	String noteStr = note(RX)
	noteStr = ReplaceNumberByKey("Xlo",noteStr,Xlo,"=")
	noteStr = ReplaceNumberByKey("Xhi",noteStr,Xhi,"=")
	noteStr = ReplaceNumberByKey("Hlo",noteStr,Hlo,"=")
	noteStr = ReplaceNumberByKey("Hhi",noteStr,Hhi,"=")
	noteStr = ReplaceNumberByKey("Flo",noteStr,Flo,"=")
	noteStr = ReplaceNumberByKey("Fhi",noteStr,Fhi,"=")
	noteStr = ReplaceNumberByKey("resolution",noteStr,resolution,"=")

	Variable N3 = NX*NH*NF							// total number of points
	printf "Interpolating Rodriques vectors, with resolution = %gm\r",resolution
	printf "interpolating on a grid [%g,%g] [%g,%g] [%g,%g], NX=%d,  NH=%d,  NF=%d\r",Xlo,Xhi,Hlo,Hhi,Flo,Fhi,NX,NH,NF
	printf "total number of points in volume = (NX*NH*NF) = %d\r",N3
	Variable predicted = ceil(0.000836*N^1.19 + 3.56e-6*N3* N^1.07)	// predicted execution time (seconds)
	//	DoAlert 1,"Interpolating could take ~"+num2istr(ceil(predicted/60))+"min at 2GhZ, do it?"
	DoAlert 1,"Interpolating could take ~"+Secs2Time(ceil(predicted),5,0)+"  at 2GhZ, do it?"
	if (V_flag>1)
		printf "apparently %s was too long to wait for interpolation\r",Secs2Time(ceil(predicted),5,0)
		return 1
	endif
	Variable now = dateTime
	printf "started at %s,   should finish at %s\r",Secs2Time(now,1),Secs2Time(now+predicted,1)
	DoUpdate

	Make/N=(N,4)/O triang4X
	triang4X[][0] = XX[p]
	triang4X[][1] = HH[p]
	triang4X[][2] = FF[p]
	triang4X[][3] = RX[p]								// Rodriques X values plus XHF
	Duplicate/O triang4X triang4H
	triang4H[][3] = RH[p]								// Rodriques H values plus XHF
	Duplicate/O triang4X triang4F
	triang4F[][3] = RF[p]								// Rodriques F values plus XHF
	Variable timer1=startMSTimer, timer2=startMSTimer
	Triangulate3d triang4X
	Wave M_3DVertexList=M_3DVertexList				// vertex list is the same for all three, since the xyz are the same
	//	printf "to triangulate the Rodriques vectors took %s\r",num2sexigesmal(stopMSTimer(timer1)*1e-6,0)
	printf "to triangulate the Rodriques vectors took %s\r",Secs2Time(stopMSTimer(timer1)*1e-6,5,0)

	Interpolate3D /RNGX={Xlo,resolution,NX}/RNGY={Hlo,resolution,NH}/RNGZ={Flo,resolution,NF}/DEST=R3dX triangulationWave=M_3DVertexList, srcWave=triang4X
	Interpolate3D /RNGX={Xlo,resolution,NX}/RNGY={Hlo,resolution,NH}/RNGZ={Flo,resolution,NF}/DEST=R3dH triangulationWave=M_3DVertexList, srcWave=triang4H
	Interpolate3D /RNGX={Xlo,resolution,NX}/RNGY={Hlo,resolution,NH}/RNGZ={Flo,resolution,NF}/DEST=R3dF triangulationWave=M_3DVertexList, srcWave=triang4F
	SetScale/I x xlo,xhi, "m", R3dX,R3dH,R3dF		// components of the interpolated Rodriques vectors interpolated into new volume
	SetScale/I y hlo,hhi, "m", R3dX,R3dH,R3dF
	SetScale/I z flo,fhi, "m", R3dX,R3dH,R3dF
	Note/K R3dX	;	Note R3dX, noteStr
	Note/K R3dH	;	Note R3dH, noteStr
	Note/K R3dF	;	Note R3dF, noteStr
	Variable/G CutResolution = resolution
	R3dX = numtype(R3dX[p][q][r]) ? 0 : R3dX[p][q][r]	// set all bad numbers to zero
	R3dH = numtype(R3dH[p][q][r]) ? 0 : R3dH[p][q][r]
	R3dF = numtype(R3dF[p][q][r]) ? 0 : R3dF[p][q][r]
	beep
	//	printf "the whole interpolation process took %s\r",num2sexigesmal(stopMSTimer(timer2)*1e-6,0)
	printf "the whole interpolation process took %s\r",Secs2Time(stopMSTimer(timer2)*1e-6,5,0)
	KillWaves/Z triang4X,triang4H,triang4F,M_3DVertexList
	return 0
End




// makes pretend test data for testing
Function MakeTestData(type)
	String type
	Prompt type,"type of test data",popup,"X;H;F;circulating;toroidal"
	DoPrompt "type",type
	if (V_flag)
		return 1
	endif
	printf "   MakeTestData(\"%s\")\r",type
	NewDataFolder/O/S root:testData
	Make/N=(51,51,51)/O R3dX,R3dH,R3dF

	SetScale/I x -25,25,"m", R3dF,R3dH,R3dX
	SetScale/I y -25,25,"m", R3dF,R3dH,R3dX
	SetScale/I z 0,50,"m", R3dF,R3dH,R3dX
	//	String noteStr = "X0=0;H0=0;F0=0;dX=0;dH=0;dF=-1;Xlo=-25;Xhi=25;Hlo=-25;Hhi=25;Fhi=50;Flo=0;resolution=1;"
	String noteStr = "X0=0;H0=0;F0=0;dX=0;dH=0;dF=1;Xlo=-25;Xhi=25;Hlo=-25;Hhi=25;Fhi=50;Flo=0;resolution=1;"
	Note/K R3dX	;	Note R3dX, noteStr
	Note/K R3dH	;	Note R3dH, noteStr
	Note/K R3dX	;	Note R3dX, noteStr
	Note/K R3dF	;	Note R3dF, noteStr

	Variable angle = tan(.5)
	Duplicate/O R3dX, tempPhi, tempDist
	Variable f0=20,r0=10
	tempPhi = atan2(y,x)
	tempDist = sqrt((x-r0*cos(tempPhi))^2+(y-r0*sin(tempPhi))^2 + (z-f0)^2)

	strswitch(type)
		case "X":
			R3dX = angle
			R3dH = 0
			R3dF = 0
			break
		case "H":
			R3dX = 0
			R3dH = angle
			R3dF = 0
			break
		case "F":
			R3dX = 0
			R3dH = 0
			R3dF = angle
			break
		case "circulating":
			R3dX = 0
			R3dH = 0
			R3dF = angle*exp(-tempDist/7)
			break
		case "toroidal":
			R3dX = -angle*sin(tempPhi)*exp(-tempDist/7)				// direction of R is same as theta^
			R3dH = angle*cos(tempPhi)*exp(-tempDist/7)
			R3dF = 0
			break
		default:
			Abort "illegal type, '"+type+"'"
	endswitch
	KillWaves/Z tempPhi, tempDist
	DoAlert 0, "Current data folder is now 'root:testData:'"
End



// start of readin process
Function/T readInXYZorientationsFromFile()
	String/G root:rawDataFullPathName
	SVAR rawDataFullPathName=root:rawDataFullPathName
	Variable refNum

	Open/R/Z=1 refNum rawDataFullPathName	// see if rawDataFullPathName exists?
	if (refNum>0)
		Close refNum
	endif

	if (!V_flag)								// V_flag==0 if file exists
		DoAlert 1, "use: "+rawDataFullPathName
		V_flag = (V_flag==2)
	endif
	if (V_flag)
		rawDataFullPathName = ""
	endif
	Open/R/M="Wirescan peak positions from Wenge"/P=home/Z=2 refNum rawDataFullPathName
	if (refNum<=0)
		Abort "unable to re-open the data file"
	endif
	rawDataFullPathName = S_fileName		// save file name for future use

	String line								// determine if this file is a new or old type
	FReadLine refNum, line
	FSetPos refNum,0
	Close refNum
	String noteStr=""
	if (char2num(line[0])==char2num("$"))
		noteStr = readInNewRawDataFile(rawDataFullPathName)	//	new
	else
		Abort "You have chosen an unknown type of file, it is not supported"
	endif
	if (strlen(noteStr)<1)
		DoAlert 0, "Unable to properly read "+rawDataFullPathName
		return ""
	endif
	return noteStr
End


// read in data from file, and create arrays XX,HH,FF of the positions of each voxel, and (RX,RH,RF) the Rodriques vector for each voxel
// the positions are in m, and the Rodriques vectors are in the XHF system, and  |Rodriques|=tan(angle/2).
Function/T readInNewRawDataFile(FullFileName)
	String FullFileName

	String noteStr = readSmarterHeader(FullFileName)
	if (strlen(noteStr)<1)
		return ""
	endif

	String fldrSav= GetDataFolder(1)
	Variable refNum
	Open /R/Z=1 refNum as FullFileName
	if (strlen(S_fileName)<1 || V_flag)
		return ""
	endif
	Close refNum
	String pathSep
	if (stringmatch(IgorInfo(2),"Macintosh"))
		pathSep = ":"
	elseif (stringmatch(IgorInfo(2),"Windows"))
		pathSep = "\\"
	else
		Abort "This is neither Mac nor Win, what is this computer?"
	endif
	String fldrName = ParseFilePath(3,FullFileName, pathSep, 0, 0)
	fldrName = CleanupName(fldrName, 0)
	if (stringmatch(fldrName,"raw"))
		fldrName = UniqueName(fldrName,11,0)
	endif

	if (CheckName(fldrName,11))
		DoAlert 2, "This data folder already exists, delete the folder and re-load the data?"
		if (V_flag==1)
			KillDataFolder $fldrName
		elseif (V_flag==2)
			fldrName = UniqueName(fldrName,11,0)
		else
			// Close refNum
			return ""
		endif
	endif
	NewDataFolder /O/S $fldrName

	// 1st three columns are X Y Z		X is PM500, so change X -> -X, Y and Z are in sample coordinates, so no negative signs
	// matij are in Wenge sample coordinates (X, -H, -F).  So negate the H and F coordinates when computing Rodriques vector
	String columnInfoStr = ""
	columnInfoStr += "N=XX;N=YY;N=ZZ;"
	columnInfoStr += "N=mat11;N=mat12;N=mat13;"
	columnInfoStr += "N=mat21;N=mat22;N=mat23;"
	columnInfoStr += "N=mat31;N=mat32;N=mat33;"
	LoadWave/G/B=columnInfoStr/Q/A/L={0,0,0,0,12} FullFileName
	if (V_flag!=12)
		SetDataFolder fldrSav
		return ""
	endif
	Wave XX=XX,YY=YY,ZZ=ZZ					// PM500 positions (m)
	Wave mat11=mat11,mat12=mat12,mat13=mat13
	Wave mat21=mat21,mat22=mat22,mat23=mat23
	Wave mat31=mat31,mat32=mat32,mat33=mat33
	Variable N=numpnts(XX)
	Make/N=(N) RX,RH,RF,totalAngles
	Make/N=3/O/D vec3
	Make/N=(3,3)/O/D mat3, refMat, refMatInv
	printf "read in %d voxels of raw data from   '%s'\r",N,FullFileName

	String list = StringByKey("refMat",noteStr,"=")
	refMat[0][0] = str2num(StringFromList(0,list,","))
	refMat[0][1] = str2num(StringFromList(1,list,","))
	refMat[0][2] = str2num(StringFromList(2,list,","))
	refMat[1][0] = str2num(StringFromList(3,list,","))
	refMat[1][1] = str2num(StringFromList(4,list,","))
	refMat[1][2] = str2num(StringFromList(5,list,","))
	refMat[2][0] = str2num(StringFromList(6,list,","))
	refMat[2][1] = str2num(StringFromList(7,list,","))
	refMat[2][2] = str2num(StringFromList(8,list,","))
	if (SquareUpMatrix(refMat))
		DoAlert 0, "refMat is invalid.  This data is bad!"
		SetDataFolder fldrSav
		return ""
	endif
	MatrixOp/O refMatInv=Inv(refMat)

	Variable i, angle
	for (i=0;i<N;i+=1)
		mat3[0][0] = mat11[i]
		mat3[0][1] = mat12[i]
		mat3[0][2] = mat13[i]
		mat3[1][0] = mat21[i]
		mat3[1][1] = mat22[i]
		mat3[1][2] = mat23[i]
		mat3[2][0] = mat31[i]
		mat3[2][1] = mat32[i]
		mat3[2][2] = mat33[i]
		SquareUpMatrix(mat3)
		MatrixOp/O mat3 = refMatInv x mat3
		angle = axisOfMatrix(mat3,vec3)		// returned angle (degrees)
		vec3 *= tan(angle*PI/180/2)			// this is now the Rodriques vector
		RX[i] = vec3[0]
		RH[i] = -vec3[1]						// Wenge reversed definition of F and H for the matricies
		RF[i] = -vec3[2]
		totalAngles[i] = angle
	endfor
	KillWaves/Z refMatInv

	Variable iroot2=1/sqrt(2)
	Duplicate/O XX,HH,FF						// make XX,HH,FF in the XHF system (see note above)
	//	XX = -XX								//	X is in sample coordinates, so X==X
	HH = (YY+ZZ)*iroot2						//	H = (Y + Z)/sqrt(2)	YY and ZZ are sample system in the file
	FF = (-YY + ZZ)*iroot2					//	F = (-Y +Z)/sqrt(2)

	Variable X0, Y0,Z0, H0, F0,  dX, dY, dZ, dH, dF
	X0 = NumberByKey("X0",noteStr,"=")
	Y0 = NumberByKey("Y0",noteStr,"=")
	Z0 = NumberByKey("Z0",noteStr,"=")
	if (numtype(X0+Y0+Z0))					// X0, Y0, or Z0 not in file, so use center of region as center
		WaveStats/Q XX
		X0 = V_avg
		WaveStats/Q HH
		H0 = V_avg
		WaveStats/Q FF
		F0 = V_avg
	else
		//	X0 = -X0							//	X0 in file is PM500 posiition
		H0 = (Y0+Z0)*iroot2					//	H = (Y + Z)/sqrt(2),	Y0 and Z0 are already sample frame
		F0 = (-Y0 + Z0)*iroot2				//	F = (-Y +Z)/sqrt(2)
	endif
	dX = NumberByKey("dX",noteStr,"=")
	dY = NumberByKey("dY",noteStr,"=")
	dZ = NumberByKey("dZ",noteStr,"=")
	if (numtype(dX+dY+dZ))
		dX = 0									// vector (dX,dY,dZ) not present, use default
		dY = -iroot2
		dZ = iroot2
	endif
	//	dX = -dX								// in file dX is PM500 position, so reverse
	dH = (dY + dZ)*iroot2						// dY and dZ are in sample system in file
	dF = (-dY +dZ)*iroot2
	vec3 = {dX,dH,dF}
	normalize(vec3)
	dX = vec3[0]
	dH = vec3[1]
	dF = vec3[2]
	noteStr = ReplaceNumberByKey("dX",noteStr,dX,"=")
	noteStr = ReplaceNumberByKey("dH",noteStr,dH,"=")
	noteStr = ReplaceNumberByKey("dF",noteStr,dF,"=")

	// center coordinates at the point {X0,H0,F0}.  This resets the origin to {0,0,0}
	XX -= X0
	HH -= H0
	FF -= F0
	printf "re-set origin of data to XHF={%g, %g, %g},   so now {0,0,0} is center\r",X0,H0,F0
	X0 = 0
	H0 = 0
	F0 = 0
	noteStr = ReplaceNumberByKey("X0",noteStr,X0,"=")	// save new X0, H0, F0
	noteStr = ReplaceNumberByKey("H0",noteStr,H0,"=")
	noteStr = ReplaceNumberByKey("F0",noteStr,F0,"=")
	noteStr = RemoveByKey("Y0", noteStr,"=")				// remove the old Y0,Z0, dY, dZ
	noteStr = RemoveByKey("Z0", noteStr,"=")
	noteStr = RemoveByKey("dY", noteStr,"=")
	noteStr = RemoveByKey("dZ", noteStr,"=")
	Note/K XX  ;  Note XX, noteStr
	Note/K HH  ;  Note HH, noteStr
	Note/K FF  ;  Note FF, noteStr
	Note/K RX  ;  Note RX, noteStr
	Note/K RH  ;  Note RH, noteStr
	Note/K RF  ;  Note RF, noteStr

	KillWaves/Z YY,ZZ
	KillWaves/Z mat11,mat12,mat13,mat21,mat22,mat23,mat31,mat32,mat33
	KillWaves/Z mat3, vec3
	SetDataFolder fldrSav
	return noteStr
End


// readin the header part
Function/T readSmarterHeader(fullFileName)
	String fullFileName
	Variable refNum
	Open /R/M="Wirescan peak positions from Wenge"/P=home/Z=1 refNum as fullFileName
	if (strlen(S_fileName)<1 || V_flag)
		return ""
	endif
	if (!stringmatch(findTagInFile(refNum,"filetype"),"ArrayOf3dOrientsFile"))
		Close refNum
		return ""												// this file is wrong type
	endif

	String noteStr = ReplaceStringByKey("refMat","","1,0,0,0,1,0,0,0,1","=")// default ref matrix
	noteStr = fetchAndAddNumber2List(refNum,"X0",noteStr,1)		// X-coord of cylindrical symmetry axis (m)
	noteStr = fetchAndAddNumber2List(refNum,"Y0",noteStr,1)		// Y-coord of cylindrical symmetry axis (m)
	noteStr = fetchAndAddNumber2List(refNum,"Z0",noteStr,1)		// Z-coord of cylindrical symmetry axis (m)
	noteStr = fetchAndAddNumber2List(refNum,"dX",noteStr,1)		// X component of unit vector in direction of cylindrical axis, beam line coordinates
	noteStr = fetchAndAddNumber2List(refNum,"dY",noteStr,1)		// Y component of unit vector in direction of cylindrical axis
	noteStr = fetchAndAddNumber2List(refNum,"dZ",noteStr,1)		// Z component of unit vector in direction of cylindrical axis
	noteStr = fetchAndAddSring2List(refNum,"refMat",noteStr)		// reference matrix identifies zero angle
	Close refNum
	return noteStr
End


// find the step size in a coordinate by examining the values
Function FindScalingFromVec(vec,threshold,first,stepSize,dimN)
	Wave vec
	Variable threshold			// a change greater than this is intentional (less is jitter)
	Variable &first				// use 	SetScale/P x first,stepSize,"",waveName
	Variable &stepSize
	Variable &dimN

	Variable last,i, Ndim=1
	Duplicate/O vec testdim
	Variable N = numpnts(testdim)
	SetScale/P x 0,1,"", testdim
	Sort testDim, testDim

	Variable delta
	stepSize=Inf
	for (i=0;i<(N-1);i+=1)
		delta = testDim[i+1]-testDim[i]
		if (delta<stepSize && delta>threshold)
			stepSize = delta
		endif
	endfor
	if (numtype(stepSize)==1)				// no steps, all are the same
		WaveStats/M=1/Q testDim
		first = V_avg
		stepSize = 0
		dimN=1
		return 0
	endif
	// stepSize is now approximately right

	Variable Nj, stepSum=0
	for (i=0;i<(N-1);i+=1)
		delta = testDim[i+1]-testDim[i]
		if (round(delta/stepSize)==1)
			stepSum += delta
			Nj += 1
		endif
	endfor
	if (Nj>0)
		stepSize = stepSum/Nj				// this is now a good average step size
	endif

	i = floor(BinarySearchInterp(testDim, testDim[0]+threshold))
	first = faverage(testDim,0,i)
	if (ItemsInList(GetRTStackInfo(0))<=1)
		print stepSize, first
	endif

	i = ceil(BinarySearchInterp(testDim, testDim[N-1]-threshold))
	dimN = round((faverage(testDim,i,N-1)-first)/stepSize + 1)
	KillWaves/Z testdim
	return 0
End



// retruns axis name assuming vec is in XHF coordinates
Function/S axisName(vec)
	Wave vec
	Wave axis=$MakeUnique3Vector(vec)	// normalized version of vec
	normalize(axis)
	Wave hat=$MakeUnique3Vector($"")
	Variable one=cos(0.1*PI/180)	// almost one (0.1 degree off)
	Variable dot

	hat = {1,0,0}
	dot = MatrixDot(axis,hat)
	if (abs(dot)>one)
		if (dot>0)
			KillWaves/Z axis,hat
			return "X"
		else
			KillWaves/Z axis,hat
			return "-X"
		endif
	endif

	hat = {0,1,0}
	dot = MatrixDot(axis,hat)
	if (abs(dot)>one)
		if (dot>0)
			KillWaves/Z axis,hat
			return "H"
		else
			KillWaves/Z axis,hat
			return "-H"
		endif
	endif

	hat = {0,0,1}
	dot = MatrixDot(axis,hat)
	if (abs(dot)>one)
		if (dot>0)
			KillWaves/Z axis,hat
			return "F (depth)"
		else
			KillWaves/Z axis,hat
			return "-F (depth)"
		endif
	endif

	hat = {0,1,-1}			// Y direction,  Y = (H-F)/sqrt(2)
	normalize(hat)
	dot = MatrixDot(axis,hat)
	if (abs(dot)>one)
		if (dot>0)
			KillWaves/Z axis,hat
			return "Y"
		else
			KillWaves/Z axis,hat
			return "-Y"
		endif
	endif

	hat = {0,1,1}			// Z direction,  Z = (H+F)/sqrt(2)
	normalize(hat)
	dot = MatrixDot(axis,hat)
	if (abs(dot)>one)
		if (dot>0)
			KillWaves/Z axis,hat
			return "Z"
		else
			KillWaves/Z axis,hat
			return "-Z"
		endif
	endif

	// not one of the standard directions, so just give direction
	String str
	sprintf str, "along (%.3g, %.3g, %.3g)",axis[0],axis[1],axis[2]
	KillWaves/Z axis,hat
	return str
End


// make a unit vector in the direction indicated by the string 'direction'
// returns the full name of the new unit vector, returns "" for invalid input
Function/S MakeDirectionVector(direction)		// make a unit vector in the specified direction
	String direction

	Wave normal = $MakeUnique3Vector($"")	// normal in the requested direction
	Variable iroot2 = 1/sqrt(2)
	strswitch(direction)
		case "X":
			normal={1,0,0}
			break
		case "H":
			normal={0,1,0}
			break
		case "F":
			normal={0,0,1}
			break
		case "Y":
			normal={0,iroot2,-iroot2}
			break
		case "Z":
			normal={0,iroot2,iroot2}
			break
		case "phiZ":
			Variable phi = NumVarOrDefault("phiSlice",0)*PI/180
			normal[0] = cos(phi)				// for a phi cut around F
			normal[1] = sin(phi)
			normal[2] = 0
			break
		case "R_Cyl":							// for cylindrical coordinates, need to recalc directions for each point
		case "Phi_Cyl":
		case "Axis_Cyl":
			normal = NaN
			break
		default:
			DoAlert 0, "Illegal direction vector ''"+direction+"'"
			KillWaves/Z normal
			return ""
	endswitch
	return GetWavesDataFolder(normal,2)
End


// fix up a matrix so that it is exactly a rotation matrix, not just close to one
Function SquareUpMatrix(mat)
	Wave mat

	Wave vec=$MakeUnique3Vector($"")
	Variable len
	Variable err

	vec[] = mat[0][p]
	len = norm(vec)
	mat[0][] = vec[q]/len
	err = len

	vec[] = mat[1][p]
	len = norm(vec)
	mat[1][] = vec[q]/len
	err += len

	vec[] = mat[2][p]
	len = norm(vec)
	mat[2][] = vec[q]/len
	err += len

	KillWaves/Z vec
	return numtype(err)
End


// ==================================================================
// =============== Start of utility section, very general stuff =====================

// find tagged numerical value in file, and return it
Function/T fetchAndAddNumber2List(refNum,tagStr,list,scaleFactor)
	Variable refNum
	String tagStr
	String list
	Variable scaleFactor			// a scale factor to apply to numbers, if NaN, use 1
	scaleFactor = numtype(scaleFactor) ? 1 : scaleFactor

	Variable value
	FSetPos refNum,0
	value = str2num(findTagInFile(refNum,tagStr))*scaleFactor
	if (numtype(value)!=2)
		list = ReplaceNumberByKey(tagStr,list,value,"=")
	endif
	return list
End


// find tagged string in file, and return it
Function/T fetchAndAddSring2List(refNum,tagStr,list)
	Variable refNum
	String tagStr
	String list

	String str
	FSetPos refNum,0
	str = findTagInFile(refNum,tagStr)
	if (strlen(str)>1 || (strlen(str)==1 && char2num(str[0])!=32))
		list = ReplaceStringByKey(tagStr,list,str,"=")
	endif
	return list
End


// return as a string the tagged value in file
// this is used by fetchAndAddNumber2List() and fetchAndAddSring2List()
Function/T findTagInFile(fileRef,tagStr)	// returns the 'value' string associated with the tag
	Variable fileRef				// ref to an opened file
	String tagStr				// the tag to search for

	String line
	tagStr = "$"+tagStr
	Variable i,c, tagLen=strlen(tagStr)
	do
		FReadLine fileRef, line
		i = strsearch(line,tagStr,0,2)
		i = (i==0 && char2num(line[tagLen])>32) ? -1 : i		// in case tag is a substring of line
	while (i && strlen(line)>0)

	if (strlen(line)<1)								// end of file
		return ""
	endif

	// find index to first useful character
	i = strlen(tagStr)-1
	do
		i += 1
		c = char2num(line[i])
	while (c<=32 || c==char2num("="))
	line = line[i,Inf]

	if (char2num(line[0])==char2num("'"))		// a quoted string
		i = strsearch(line,"'",1)
		if (i<0)
			Abort "encountered a partially quoted string in tagged data file"
		endif
		line = line[1,i-1]
		return line
	endif

	// find end of value
	i = strsearch(line,"//",0)						// remove possible comment
	line = SelectString(i,line," ",line[0,i-1])
	// i<0		no comment
	// i=0		only a comment, so use 1 space
	// i>0		comment found, remove it

	// remove trailing white space
	i = strlen(line)
	do
		i -= 1
		c = char2num(line[i])
	while(c<=32 && i>0)
	line = line[0,i]
	line = SelectString(strlen(line)==0,line," ")	// empty string means EOF, so make contents at least one space
	return line
End


Static Function normalize(a)	// normalize a and return the initial magnitude, also in Utility_JZT.ipf
	Wave a
	Variable norm_a = norm(a)
	a /= norm_a
	return norm_a
End

// ================ End of utility section, very general stuff =====================
// ==================================================================





// *************************************************************************************
// *************************************************************************************
// *************************************************************************************
// ********************************* Start of Grain Math Items *********************************

Function rotationAngleOfMat(rot)				// returns the total rotation angle of a matrix 'rot'
	Wave rot									// the rotation matrix
	Variable trace = MatrixTrace(rot)			// trace = 1 + 2*cos(theta)
	Variable cosine = (trace-1)/2				// cosine of the rotation angle
	cosine = (cosine>1) ? (2-cosine) : cosine
	return acos(cosine)*180/PI				// rotation angle in degrees
End


// compute angle and axis of a rotation matrix
Function axisOfMatrix(rot,axis)
	// returns total rotation angle, and sets axis to the axis of the total rotation
	Wave rot									// rotation matrix
	Wave axis									// axis of the rotation (angle is returned)

	Make/N=(3,3)/O/D axisMat__
	axis = rot[p][0]
	normalize(axis)
	axisMat__[][0] = axis[p]

	axis = rot[p][1]
	normalize(axis)
	axisMat__[][1] = axis[p]

	axis = rot[p][2]
	normalize(axis)
	axisMat__[][2] = axis[p]

	Variable cosine = (MatrixTrace(axisMat__)-1)/2	// trace = 1 + 2*cos(theta)
	cosine = max(-1,min(cosine,1))
	if (cosine<= -1)							// special for 180 rotation,
		axis[0] = sqrt((axisMat__[0][0]+1)/2)
		axis[1] = sqrt((axisMat__[1][1]+1)/2)
		axis[2] = sqrt((axisMat__[2][2]+1)/2)	// always assume z positive
		axis[0] = (axisMat__[0][2]+axisMat__[2][0])<0 ? -axis[0] : axis[0]
		axis[1] = (axisMat__[1][2]+axisMat__[2][1])<0 ? -axis[1] : axis[1]
	else											// rotaion < 180, usual formula works
		axis[0] = axisMat__[1][2] - axisMat__[2][1]
		axis[1] = axisMat__[2][0] - axisMat__[0][2]
		axis[2] = axisMat__[0][1] - axisMat__[1][0]
	endif
	normalize(axis)
	KillWaves /Z axisMat__
	return acos(cosine)*180/PI						// rotation angle in degrees
End


// set mat to be a rotation matrix about axis with angle
Function rotationMatAboutAxis(axis,angle,mat)
	Wave axis				// axis about which to rotate
	Variable angle			// angle to rotate (passed as degrees, but convert to radians)
	Wave mat				// desired rotation matrix
	angle *= PI/180

	Make/N=3/O/D xhat_rotate__, yhat_rotate__, zhat_rotate__
	Wave xhat=xhat_rotate__
	Wave yhat=yhat_rotate__
	Wave zhat=zhat_rotate__

	zhat = axis
	if (normalize(zhat) <=0)
		KillWaves/Z xhat_rotate__, yhat_rotate__, zhat_rotate__
		return 1			// error, cannot rotate about a zero length axis
	endif

	Variable i
	i = ( abs(zhat[0])<= abs(zhat[1]) ) ? 0 : 1
	i = ( abs(zhat[2])< abs(zhat[i]) ) ? 2 : i
	xhat = zhat
	xhat[i] = 2							// choose x-z plane
	normalize(xhat)
	Variable dotxz = MatrixDot(xhat,zhat)
	xhat -= dotxz*zhat					// xhat is now perpendicular to zhat
	normalize(xhat)					// xhat is now normalized vector perp to zhat
	crossV4(zhat,xhat,yhat)
	normalize(yhat)					// yhat is now normalized vector perp to zhat and xhat
//	xhat, yhat, zhat is now an orthonormal system with zhat || to axis

//	now rotate around zhat in the xhat -> yhat direction, an angle of angle
	Variable cosa = cos(angle)
	Variable sina = sin(angle)
	Redimension/N=(3,3) mat
	for(i=0;i<3;i+=1)
		mat[][i] = (xhat[i]*cosa+yhat[i]*sina)*xhat[p] + (-xhat[i]*sina + yhat[i]*cosa)*yhat[p] + zhat[i]*zhat[p]
	endfor
	KillWaves/Z xhat_rotate__, yhat_rotate__, zhat_rotate__
End
// cross supplanted by the Cross operation in Igor 5
Static Function crossV4(a,b,c)	// vector cross product
	Wave a,b,c
	c[2] = a[0]*b[1] - a[1]*b[0]
	c[0] = a[1]*b[2] - a[2]*b[1]
	c[1] = a[2]*b[0] - a[0]*b[2]
End


// rotate mat about x, y, or z by angle, not a general axis
Function rotateMat(mat,axis,angle)
	Wave mat				// matrix to rotate
	Variable axis			// axis to rotate about 0=x, 1=y, 2=z
	Variable angle			// angle to rotate (radians)

	Variable cosa = cos(angle)
	Variable sina = sin(angle)
	Make/N=(3,3)/O mat_rot_
	mat_rot_ = (p==q)

	switch(axis)
		case 0:			// rotate about x-axis
			mat_rot_[1][1] = cosa
			mat_rot_[2][2] = cosa
			mat_rot_[1][2] = -sina
			mat_rot_[2][1] =   sina
			break
		case 1:			// rotate about y-axis
			mat_rot_[0][0] = cosa
			mat_rot_[2][2] = cosa
			mat_rot_[0][2] =   sina
			mat_rot_[2][0] = -sina
			break
		case 2:			// rotate about z-axis
			mat_rot_[0][0] = cosa
			mat_rot_[1][1] = cosa
			mat_rot_[0][1] = -sina
			mat_rot_[1][0] =   sina
			break
		default:
	endswitch
	Killwaves/Z mat_rot_
End

// ********************************** End of Grain Math Items *********************************
// *************************************************************************************
// *************************************************************************************
// *************************************************************************************

